@@ -32,6 +32,8 @@
 #' @export
 sensitivitiesSymb <- function(f, states = names(f), parameters = NULL, inputs = NULL, events = NULL, reduce = FALSE) {
   
+  # If f contains line breaks, replace them by ""
+  f <- gsub("\n", "", f)
   
   variables <- names(f)
   states <- states[!states%in%inputs]
@@ -39,7 +41,8 @@ sensitivitiesSymb <- function(f, states = names(f), parameters = NULL, inputs =
   if (is.null(parameters)) {
     pars <- getSymbols(c(f,
                          as.character(events[["value"]]),
-                         as.character(events[["time"]])),
+                         as.character(events[["time"]]),
+                         as.character(events[["root"]])),
                        exclude = c(variables, inputs, "time"))
   } else {
     pars <- parameters[!parameters%in%inputs]
@@ -91,6 +94,8 @@ sensitivitiesSymb <- function(f, states = names(f), parameters = NULL, inputs =
   events.addon <- eventframe <- NULL
   if (!is.null(events)) {
     
+    if (is.null(events[["root"]])) events[["root"]] <- NA
+    
     events.addon <- lapply(1:nrow(events), function(i) {
       
       # A data frame representing the i'th event
@@ -100,8 +105,11 @@ sensitivitiesSymb <- function(f, states = names(f), parameters = NULL, inputs =
       xone <- as.character(myevent[["var"]])
       tau <- as.character(myevent[["time"]])
       xi <- as.character(myevent[["value"]])
+      root <- as.character(myevent[["root"]])
       xk <- setdiff(variables, xone)
-      
+      rootstate <- intersect(getSymbols(root), variables)[1]
+      norootstate <- setdiff(variables, rootstate)
+      rootpar <- intersect(getSymbols(root), pars)[1]
       
       # Derive some quantities needed for all event methods
       Ji1 <- jacobianSymb(f, variables = xone)
@@ -109,6 +117,7 @@ sensitivitiesSymb <- function(f, states = names(f), parameters = NULL, inputs =
       Jk1 <- Ji1[paste0(xk, ".", xone)]
       f1 <- f[xone]
       fk <- f[xk]
+      fr <- f[rootstate]
       
       # Collect ODE parameters 
       odepars <- c(states, pars)
@@ -121,45 +130,103 @@ sensitivitiesSymb <- function(f, states = names(f), parameters = NULL, inputs =
         d <- list()
         
         vars <- intersect(paste0(xone, ".", odepars), newvariables)
-        if (length(vars) > 0) {
+        if (length(vars) > 0 & is.na(root)) {
           d[[1]] <- data.frame(
             var = vars,
             time = tau,
             value = 0,
+            root = root,
             method = "replace"
           )
         }
         
-        vars <- intersect(paste0(xone, ".", tau), newvariables)
-        if (length(vars) > 0) {
+        # Resetting of tau sensitivities for multiple roots
+        vars <- intersect(paste0(c(xone, xk), ".", tau), newvariables)
+        if (length(vars) > 0 & !is.na(root)) {
           d[[2]] <- data.frame(
+            var = vars,
+            time = tau,
+            value = 0,
+            root = root,
+            method = "replace"
+          )
+        }
+        
+        
+        vars <- intersect(paste0(xone, ".", tau), newvariables)
+        if (length(vars) > 0) {
+          d[[3]] <- data.frame(
             var = vars,
             time = tau,
             value = paste0("(", J11, ")*(", xone, "- (", xi, ")) - (", f1, ")"),
+            root = root,
             method = "add"
           )
         }
           
         vars <- intersect(paste0(xk, ".", tau), newvariables)
         if (length(vars) > 0) {
-          d[[3]] <- data.frame(
+          d[[4]] <- data.frame(
             var = vars,
             time = tau,
             value = paste0("(", Jk1, ")*(", xone, "- (", xi, "))"),
+            root = root,
             method = "add"
           )
         }
         
         vars <- intersect(paste0(xone, ".", xi), newvariables)
         if (length(vars) > 0) {
-          d[[4]] <- data.frame(
+          d[[5]] <- data.frame(
             var = vars,
             time = tau,
             value = 1,
+            root = root,
             method = "add"
           )
         }
         
+        
+        vars <- intersect(paste0(c(xone, xk), ".", rootpar), newvariables)
+        if (length(vars) > 0) {
+          d[[6]] <- do.call(rbind, mapply(function(mystate, mypar) {
+            data.frame(
+              var = paste0(mystate, ".", mypar),
+              time = tau,
+              value = paste0("(eventcounter__ + 1)*", mystate, ".", tau , "/(", fr, ")"),
+              root = root,
+              method = "replace"
+            )
+          }, 
+          mystate = sapply(vars, function(v) strsplit(v, ".", fixed = TRUE)[[1]][1]),
+          mypar =  sapply(vars, function(v) strsplit(v, ".", fixed = TRUE)[[1]][2]),
+          SIMPLIFY = FALSE))
+          d[[6]] <- d[[6]][order(d[[6]][["method"]], decreasing = TRUE), ]
+        }
+   
+        excludedPars <- NULL
+        if (!is.na(root)) excludedPars <- c(rootpar, tau, xi)
+        vars <- outer(c(xone, xk), setdiff(odepars, excludedPars), function(x, y) paste(x, y, sep = "."))
+        vars <- intersect(vars, newvariables)
+        # sort first non-root states then root state
+        vars <- c(vars[!grepl(paste0("^", rootstate, "\\."), vars)], vars[grepl(paste0("^", rootstate, "\\."), vars)])
+        if (length(vars) > 0 & !is.na(root)) {
+          d[[7]] <- do.call(rbind, mapply(function(mystate, mypar) {
+            data.frame(
+              var = paste0(mystate, ".", mypar),
+              time = tau,
+              value = paste0(mystate, ".", tau, "*(-", rootstate, ".", mypar, ")/(", fr, ")"), 
+              root = root,
+              method = ifelse(mystate == xone, "replace", "add")
+            )
+          }, 
+          mystate = sapply(vars, function(v) strsplit(v, ".", fixed = TRUE)[[1]][1]),
+          mypar =  sapply(vars, function(v) strsplit(v, ".", fixed = TRUE)[[1]][2]),
+          SIMPLIFY = FALSE))
+          #d[[7]] <- d[[7]][order(d[[7]][["method"]], decreasing = TRUE), ]
+        }
+        
+        
         d[["stringsAsFactors"]] <- FALSE
         
         return(do.call(rbind, d))
@@ -179,6 +246,7 @@ sensitivitiesSymb <- function(f, states = names(f), parameters = NULL, inputs =
             var = vars,
             time = tau,
             value = 0,
+            root = root,
             method = "add"
           )
         }
@@ -189,6 +257,7 @@ sensitivitiesSymb <- function(f, states = names(f), parameters = NULL, inputs =
             var = vars,
             time = tau,
             value = paste0("-(", J11, ") * (", xi, ")"),
+            root = root,
             method = "add"
           )
         }
@@ -199,6 +268,7 @@ sensitivitiesSymb <- function(f, states = names(f), parameters = NULL, inputs =
             var = vars,
             time = tau,
             value = paste0("-(", Jk1, ") * (", xi, ")"),
+            root = root,
             method = "add"
           )
         }
@@ -209,10 +279,12 @@ sensitivitiesSymb <- function(f, states = names(f), parameters = NULL, inputs =
             var = vars,
             time = tau,
             value = 1,
+            root = root,
             method = "add"
           )
         }
         
+        
         d[["stringsAsFactors"]] <- FALSE
         
         return(do.call(rbind, d))
@@ -232,6 +304,7 @@ sensitivitiesSymb <- function(f, states = names(f), parameters = NULL, inputs =
             var = vars,
             time = tau,
             value = xi,
+            root = root,
             method = "multiply" 
           )
         }
@@ -242,6 +315,7 @@ sensitivitiesSymb <- function(f, states = names(f), parameters = NULL, inputs =
             var = vars,
             time = tau,
             value = paste0("(1 - (", xi, "))*((", J11, ")*(", xone, ") - (", f1, "))"),
+            root = root,
             method = "add"
           )
         }
@@ -252,6 +326,7 @@ sensitivitiesSymb <- function(f, states = names(f), parameters = NULL, inputs =
             var = vars,
             time = tau,
             value = paste0("(1 - (", xi, "))*((", Jk1, ")*(", xone, "))"),
+            root = root,
             method = "add"
           )
         }
@@ -262,6 +337,7 @@ sensitivitiesSymb <- function(f, states = names(f), parameters = NULL, inputs =
             var = vars,
             time = tau,
             value = xone,
+            root = root,
             method = "add"
           )
         }
@@ -269,6 +345,7 @@ sensitivitiesSymb <- function(f, states = names(f), parameters = NULL, inputs =
         d[["stringsAsFactors"]] <- FALSE
         
         
+        
         return(do.call(rbind, d))
         
         
@@ -314,11 +391,26 @@ sensitivitiesSymb <- function(f, states = names(f), parameters = NULL, inputs =
     is_neutral <- (eventframe$method == "add" & values.vals == 0)
     
     # Reduce eventframe
-    eventframe <- eventframe[!is_neutral,]
+    # eventframe <- eventframe[!is_neutral,]
     
     # Reduce events (by name matching)
     events <- lapply(events, function(e) e[rownames(e) %in% rownames(eventframe), ])
     
+    # Check if any root-like events
+    # events <- lapply(events, function(e){
+    #   if (all(is.na(eventframe[["root"]]))) {
+    #     e <- e[-match("root", names(e))]
+    #   }
+    #   return(e)
+    # })
+    
+    # No factors in events
+    events <- lapply(events, function(e) {
+      e <- lapply(e, as.character)
+      e <- as.data.frame(e, stringsAsFactors = FALSE)
+      return(e)
+    })
+    
     
   }
  
